from bs4 import BeautifulSoup
import requests
import re

ALL_URL = 'https://www.laundryalert.com/cgi-bin/penn6389/LMPage?Login=True'
HALL_BASE_URL = 'https://www.laundryalert.com/cgi-bin/penn6389/LMRoom?Halls='


class Laundry(object):
    """The client for Laundry. Used to make requests to the API.

    Usage::

      >>> from penn import Laundry
      >>> l = Laundry()
    """

    def __init__(self):
        pass

    @staticmethod
    def get_hall_no(href):
        return int(re.search(r"Halls=(\d+)", href).group(1))

    def all_status(self):
        """Return names, hall numbers, and the washers/dryers available for all
        rooms in the system

        >>> all_laundry = l.all_status()
        """
        r = requests.get(ALL_URL)
        r.raise_for_status()

        parsed = BeautifulSoup(r.text)
        info_table = parsed.find_all('table')[2]

        # This bit of code generates a dict of hallname->hall number by
        # parsing the link href of each room
        hall_dict = {}
        for link in info_table.find_all('a', class_='buttlink'):
            clean_link = link.get_text().strip()
            hall_dict[clean_link] = self.get_hall_no(link.get('href'))

        # Parse the table into the relevant data
        data = []
        for row in info_table.find_all('tr'):
            row_data = (val.get_text().strip() for val in row.find_all('td'))
            clean_row = [val for val in row_data if len(val) > 0]
            data.append(clean_row)

        # Remove the header row, service row, and all empty rows
        data_improved = [row for row in data if len(row) > 0][1:]

        # Construct the final JSON
        laundry_rooms = []
        for row in data_improved:
            room_dict = dict()
            room_dict['washers_available'] = int(row[1])
            room_dict['dryers_available'] = int(row[2])
            room_dict['washers_in_use'] = int(row[3])
            room_dict['dryers_in_use'] = int(row[4])
            room_dict['hall_no'] = hall_dict[row[0]]
            room_dict['name'] = row[0]
            laundry_rooms.append(room_dict)
        return laundry_rooms

    def hall_status(self, hall_no):
        """Return the status of each specific washer/dryer in a particular
        laundry room.

        :param hall_no:
             integer corresponding to the id number for the hall. Thus number
             is returned as part of the all_status call.

        >>> english_house = l.hall_status(2)
        """
        try:
            num = int(hall_no)
        except ValueError:
            raise ValueError("Room Number must be integer")

        r = requests.get(HALL_BASE_URL + str(num))
        r.raise_for_status()

        parsed = BeautifulSoup(r.text, 'html5lib')
        tables = parsed.find_all('table')
        hall_name = tables[2].get_text().strip()
        info_table = tables[4]

        # Parse the table into the relevant data
        data = []
        for row in info_table.find_all('tr'):
            row_data = (val.get_text().strip() for val in row.find_all('td'))
            clean_row = [val for val in row_data if len(val) > 0]
            data.append(clean_row)

        # Remove the header row and all empty rows
        data_improved = [row for row in data if len(row) > 0][1:]

        def toDict(data_row):
            d = dict()
            d['number'] = data_row[0]
            d['machine_type'] = data_row[1]
            d['available'] = data_row[2] == u'Available'
            if len(data_row) == 4:
                d['time_left'] = data_row[3]
            else:
                d['time_left'] = None
            return d

        return {'machines': list(map(toDict, data_improved)), 'hall_name': hall_name}



"""A module for consuming the Penn Dining API"""
from os import path
from .base import WrapperBase


BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/dining/"
V2_BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/dining/v2/?service="

ENDPOINTS = {
    'MENUS': BASE_URL + 'menus',
    'VENUES': BASE_URL + 'venues',
}

V2_ENDPOINTS = {
    'VENUES': V2_BASE_URL + 'venues',
    'HOURS': V2_BASE_URL + 'cafes&cafe=',
    'MENUS': V2_BASE_URL + 'menus&cafe=',
    'ITEMS': V2_BASE_URL + 'items&item='
}


# Normalization for dining menu data
def normalize_weekly(data):
    if "tblMenu" not in data["result_data"]["Document"]:
        data["result_data"]["Document"]["tblMenu"] = []
    if isinstance(data["result_data"]["Document"]["tblMenu"], dict):
        data["result_data"]["Document"]["tblMenu"] = [data["result_data"]["Document"]["tblMenu"]]
    for day in data["result_data"]["Document"]["tblMenu"]:
        if "tblDayPart" not in day:
            continue
        if isinstance(day["tblDayPart"], dict):
            day["tblDayPart"] = [day["tblDayPart"]]
        for meal in day["tblDayPart"]:
            if isinstance(meal["tblStation"], dict):
                meal["tblStation"] = [meal["tblStation"]]
            for station in meal["tblStation"]:
                if isinstance(station["tblItem"], dict):
                    station["tblItem"] = [station["tblItem"]]
    return data


class DiningV2(WrapperBase):
    """The client for the Registrar. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import DiningV2
      >>> din = DiningV2('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """
    def venues(self):
        """Get a list of all venue objects.

          >>> venues = din.venues()
        """
        response = self._request(V2_ENDPOINTS['VENUES'])
        return response

    def hours(self, venue_id):
        """Get the list of hours for the venue corresponding to
        venue_id.

        :param venue_id:
          A string representing the id of a venue, e.g. "abc".


        >>> commons_hours = din.hours("593")
        """
        response = self._request(V2_ENDPOINTS['HOURS'] + venue_id)
        return response

    def menu(self, venue_id, date):
        """Get the menu for the venue corresponding to venue_id,
        on date.

        :param venue_id:
          A string representing the id of a venue, e.g. "abc".
        :param date:
          A string representing the date of a venue's menu, e.g. "2015-09-20".


        >>> commons_menu = din.menu("593", "2015-09-20")
        """
        query = "&date=" + date
        response = self._request(V2_ENDPOINTS['MENUS'] + venue_id + query)
        return response

    def item(self, item_id):
        """Get a description of the food item corresponding to item_id.

        :param item_id:
          A string representing the id of an item, e.g. "3899220".


        >>> tomato_sauce = din.item("3899220")
        """
        response = self._request(V2_ENDPOINTS['ITEMS'] + item_id)
        return response

class Dining(WrapperBase):
    """The client for the Registrar. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import Dining
      >>> din = Dining('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """
    def venues(self):
        """Get a list of all venue objects.

          >>> venues = din.venues()
        """
        response = self._request(ENDPOINTS['VENUES'])
        # Normalize `dateHours` to array
        for venue in response["result_data"]["document"]["venue"]:
            if isinstance(venue.get("dateHours"), dict):
                venue["dateHours"] = [venue["dateHours"]]
            if "dateHours" in venue:
                for dh in venue["dateHours"]:
                    if isinstance(dh.get("meal"), dict):
                        dh["meal"] = [dh["meal"]]
        return response

    def menu_daily(self, building_id):
        """Get a menu object corresponding to the daily menu for the
        venue with building_id.

        :param building_id:
          A string representing the id of a building, e.g. "abc".


        >>> commons_today = din.menu_daily("593")
        """
        response = self._request(
            path.join(ENDPOINTS['MENUS'], 'daily', str(building_id))
        )
        # Normalize `tblDayPart` and `tblItem` to array
        meals = response["result_data"]["Document"]["tblMenu"]["tblDayPart"]
        if isinstance(meals, dict):
            response["result_data"]["Document"]["tblMenu"]["tblDayPart"] = [meals]
        for meal in response["result_data"]["Document"]["tblMenu"]["tblDayPart"]:
            if isinstance(meal["tblStation"], dict):
                meal["tblStation"] = [meal["tblStation"]]
            for station in meal["tblStation"]:
                if isinstance(station["tblItem"], dict):
                    station["tblItem"] = [station["tblItem"]]
        return response

    def menu_weekly(self, building_id):
        """Get an array of menu objects corresponding to the weekly menu for the
        venue with building_id.

        :param building_id:
            A string representing the id of a building, e.g. "abc".

        >>> commons_week = din.menu_weekly("593")
        """
        response = self._request(path.join(ENDPOINTS['MENUS'], 'weekly', str(building_id)))
        return normalize_weekly(response)
